

#    MIDAS Access Client  for the  EPICSAccess service

global env
source [file join $env(MIDASBASE) SOAP tcl MIDASAccessClient.tcl]    ;#   load default support procedures

global EPICSAccessServerHost EPICSAccessServerPort

if {[info exists EPICSAccessServerHost] == 0} then {set EPICSAccessServerHost localhost}
if {[info exists EPICSAccessServerPort] == 0} then {set EPICSAccessServerPort 8015}

variable uri "urn:EPICSAccessServer"
variable proxy "http://$EPICSAccessServerHost:$EPICSAccessServerPort/EPICSAccessServer"

global MIDAS_EPICSAlarmSeverity MIDAS_EPICSAlarmStatus
set MIDAS_EPICSAlarmSeverity "Unknown"
set MIDAS_EPICSAlarmStatus   "Unknown"

global EPICS_Verbose
set EPICS_Verbose 0


#   MIDAS_EPICSAlarmSeverity
#	"NO_ALARM"
#	"MINOR_ALARM"
#	"MAJOR_ALARM"
#	"INVALID_ALARM"

#   MIDAS_EPICSAlarmStatus
#	"NO_ALARM"
#        READ_ALARM
#        WRITE_ALARM
#        HIHI_ALARM
#        HIGH_ALARM
#        LOLO_ALARM
#        LOW_ALARM
#        STATE_ALARM
#        COS_ALARM
#        COMM_ALARM
#        TIMEOUT_ALARM
#        HW_LIMIT_ALARM	
#        CALC_ALARM
#        SCAN_ALARM
#        LINK_ALARM
#        SOFT_ALARM
#        BAD_SUB_ALARM
#        UDF_ALARM
#        DISABLE_ALARM
#        SIMM_ALARM
#        READ_ACCESS_ALARM
#        WRITE_ACCESS_ALARM



proc ::SOAP::MIDAS_EPICSAccessClient_Reply {procVarName xml} {
    upvar $procVarName procvar
    global MIDAS_SOAPAccessClient_Verbose

    if {$MIDAS_SOAPAccessClient_Verbose} {
       puts "::SOAP::MIDAS_SOAPAccessClient_Reply called with $procVarName"
       puts "$xml"
       foreach member [array names procvar] {puts "procvar($member) =  $procvar($member)"}
        puts ""
    }

    if {$xml == {} && ![string match "http*" $procvar(proxy)]} {
        # This is probably not an error. SMTP and FTP won't return anything
        # HTTP should always return though (I think).
        return {}
    }

    set expected "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    if {[string compare -length 19 $xml $expected] == -1} {

#    unexpected reply or an error 

       return [::SOAP::parse_soap_response $procVarName $xml]
    }

#    This parser is VERY simple
#    Just look for the token <return and return whatever is the payload

#   insert-debug "::SOAP::MIDAS_EPICSAccessClient_Reply $xml"

#    Note - Using the Dom code to parse the XML  takes 10 ms

   global MIDAS_SOAPReturnCode MIDAS_SOAPFaultCode MIDAS_SOAPFaultString MIDAS_SOAPErrorCode MIDAS_SOAPErrorInfo
   global MIDAS_EPICSAlarmSeverity MIDAS_EPICSAlarmStatus

   set MIDAS_SOAPFaultCode   "NONE"
   set MIDAS_SOAPFaultString "NONE"
   set MIDAS_SOAPErrorCode   "NONE"
   set MIDAS_SOAPErrorInfo   "NONE"
   
   set MIDAS_SOAPReturnCode 0

   set MIDAS_EPICSAlarmSeverity "Unknown"
   set MIDAS_EPICSAlarmStatus   "Unknown"
   
   set result -1
   set i 0

    set frags [split $xml >]
    foreach frag $frags {
       if {[string compare -length 7 $frag <return] == 0 || [string compare -length 7 $frag <result] == 0} {
           set result "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           break
       }

       if {[string compare -length 5 $frag <Code] == 0} {
           set result "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
       }
       if {[string compare -length 6 $frag <State] == 0} {
           set MIDAS_EPICSAlarmSeverity "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           if {[string equal [set MIDAS_EPICSAlarmSeverity] "NO_ALARM"]} then {set MIDAS_EPICSAlarmStatus "NO_ALARM"} else {set MIDAS_EPICSAlarmStatus "WRITE_ALARM"}
       }
       if {[string compare -length 6 $frag <Cause] == 0} {
           set MIDAS_EPICSAlarmStatus "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
       }

       if {[string compare -length 10 $frag <faultcode] == 0} {
           set MIDAS_SOAPFaultCode "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 12 $frag <faultstring] == 0} {
           set MIDAS_SOAPFaultString "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 10 $frag <errorcode] == 0} {
           set MIDAS_SOAPErrorCode "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 10 $frag <errorinfo] == 0} {
           set MIDAS_SOAPErrorInfo "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       incr i
    }

    if {$MIDAS_SOAPReturnCode == 0} then {
        return $result
    } else {
# insert-debug "MIDAS_SOAPErrorCode: $MIDAS_SOAPErrorCode"
# insert-debug "MIDAS_SOAPErrorInfo: $MIDAS_SOAPErrorInfo"
# insert-debug "MIDAS_SOAPFaultCode: $MIDAS_SOAPFaultCode"
# insert-debug "MIDAS_SOAPFaultString: $MIDAS_SOAPFaultString"
      error "$MIDAS_SOAPErrorCode"  "$MIDAS_SOAPErrorInfo"  "$MIDAS_SOAPFaultCode-$MIDAS_SOAPFaultString"
        return -code error -errorcode  "$MIDAS_SOAPErrorCode" -errorinfo "$MIDAS_SOAPFaultCode-$MIDAS_SOAPFaultString"
    }
}




SOAP::create EPICSAccessClient__Verbose -proxy $proxy -uri $uri -timeout 1000 \
     -name  AccessServerVerbose \
     -params {opt int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create EPICSAccessClient__Poll -proxy $proxy -uri $uri -timeout 1000 \
     -name  Poll \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create EPICSAccessClient__GetNames -proxy $proxy -uri $uri -timeout 1000 \
     -name  GetNames \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create EPICSAccessClient__CreateChannel -proxy $proxy -uri $uri -timeout 10000 \
     -name  CreateChannel \
     -params {PVname string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create EPICSAccessClient__ClearChannel -proxy $proxy -uri $uri -timeout 10000 \
     -name  ClearChannel \
     -params {PVname string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create EPICSAccessClient__PutInt -proxy $proxy -uri $uri -timeout 10000 \
     -name  PutInt \
     -params {PVname string Value int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply

SOAP::create EPICSAccessClient__GetInt -proxy $proxy -uri $uri -timeout 10000 \
     -name  GetInt \
     -params {PVname string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply 

SOAP::create EPICSAccessClient__PutFloat -proxy $proxy -uri $uri -timeout 10000 \
     -name  PutFloat \
     -params {PVname string Value float} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply

SOAP::create EPICSAccessClient__GetFloat -proxy $proxy -uri $uri -timeout 10000 \
     -name  GetFloat \
     -params {PVname string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply  

SOAP::create EPICSAccessClient__PutString -proxy $proxy -uri $uri -timeout 10000 \
     -name  PutString \
     -params {PVname string Value string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply

SOAP::create EPICSAccessClient__GetString -proxy $proxy -uri $uri -timeout 10000 \
     -name  GetString \
     -params {PVname string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_EPICSAccessClient_Reply  


package provide EPICSAccessClient 1.0
