
set debug 0
set started 0


set frame OPT_FONTS
set frametitle "Font Selection Frame"
set icontitle "FontSel"

set savefile "$env(HOME)/.eg-tkfonts.tcl"

set font ""

set wtype 0

set fndry 1
set fmly 2
set wght 1
set pxlsz 9
set ptsz 10
set slant 2
set swdth 1
set rgstry 1

set widgets [list button checkbutton entry label listbox menubutton menu message text footer]

set fndrys [list * adobe linotype]
set fmlys  [list * courier helvetica palatino times]
set wghts  [list * medium bold]
set pxlszs [list * 0 2 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 24 25 26 30 33 34 40 50 60]
set ptszs  [list * 0 20 50 60 70 80 90 100 110 120 130 140 150 160 170 180 190 200 230 240 250 300 350 400]
set slants [list * i r]
set swdths [list * normal]
set rgstrys [list * iso8859]


proc make-frame {} {
  global fndrys fmlys wghts pxlszs ptszs slants swdths rgstrys
  global fndry fmly wght pxlsz ptsz slant swdth rgstry
  global widgets wtype

  :set footer_position center
  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
    :icon-name "[self icon-title]" :resize F

  :panel P1 :padx 10 :pady 10 :layout L
  :layout V
  :row-gap 5
  :choice-stack FNDRY  :valuex 10c :value $fndry  :label "foundry"    :strings $fndrys
  :choice-stack FMLY   :valuex 10c :value $fmly   :label "family"     :strings $fmlys
  :choice-stack WGHT   :valuex 10c :value $wght   :label "weight"     :strings $wghts
  :choice-stack PXLSZ  :valuex 10c :value $pxlsz  :label "pixel size" :strings $pxlszs
  :choice-stack PTSZ   :valuex 10c :value $ptsz   :label "point size" :strings $ptszs
  :choice-stack SLANT  :valuex 10c :value $slant  :label "slant"      :strings $slants
  :choice-stack SWDTH  :valuex 10c :value $swdth  :label "width"      :strings $swdths
  :choice-stack RGSTRY :valuex 10c :value $rgstry :label "registry"   :strings $rgstrys
 
  :panel P2 :padx 10 :pady 10 :layout L
  :layout V
  :row-gap 5
  :choice-stack WTYPE :valuex 80 :value $wtype :label "widget" :strings $widgets 

  :vskip 25
  :button RESTORE  :label "Restore Settings"
  :button SAVE     :label "Save Settings as default"

  :layout H
  :next-row T
  :button RESET    :label "Reset"
  :button APPLY    :label "Apply"
  :button REDISPLAY :label "Redisplay"

  :layout V
  :next-row T
  :vskip 30
  :button SELECT  :label "Select Font"  :w 32

  :panel P3 :padx 10 :layout B :after FOOTER
  :text FVALUE :label "font" :value "" :w 60

  :show T

}

proc paint-sample {} {

 :modify [self frame-name] :busy T

 catch {:modify S1 :delete T}
 catch {:modify S2 :delete T}
 catch {:modify S3 :delete T}

 :panel S1 :padx 10 :layout B :after FOOTER
 :layout H
 :row-gap 5; :col-gap 25
 :vskip 10

 :button SB   :label "A Sample Button"
 :slider SS   :label "A Sample Slider" :value 1 :min 0 :max 99
 :next-row T

 :message SM  :label "A Sample Message"
 :gauge SGG    :label "A Sample Gauge" :value 1 :min 0 :max 99
 :next-row T

 :number SN   :label "A Sample Number" :value 1 :w 2 :min 0 :max 99

 :panel S2 :padx 10 :pady 10 :layout B :after FOOTER
 :layout V
 :row-gap 5

 :choice-stack SCS :label "A Sample Choice-Stack" :strings [list c1 c2]
 :choice SCH  :label "A Sample Choice"  :value 1 :strings [list 1 2] 
 :menubutton SMB :label "A Sample MenuButton" :strings [list item1 item2]
 :text ST      :label "A Sample Text"     :value "Midas Forever"
 :checkbox SCB :label "A Sample Checkbox" :value 0

 global _wreq
 set gx [expr $_wreq + 20]
 :galley SG  :strings "A Sample Galley (text widget)"  :rows 7 :cols 30 :x $gx :y 0

 :panel S3 :padx 10 :layout B :after FOOTER
 :list  SL :strings [list "A Sample List (listbox widget)"] :rows 2 :cols 60

 :modify [self frame-name] :busy F

 set-footer "Sample repainted at [date]"
}

proc paint-font {} {
  global env font widgets wtype wght

  set widget [lindex $widgets $wtype]
  if {$wght == 2} then {set comp "b"} else {set comp "n"}

  global _font_[set widget]_[set comp]
  if { [info exists _font_[set widget]_[set comp]] } then {
    set font [set _font_[set widget]_[set comp]]
  } else {
    if {[info exists env(tkfont$widget$comp)]} then {
      set font $env(tkfont$widget$comp)
    } else {
      set font ""
    }
  }
  :modify FVALUE :value $font
}

proc click-font {w v} {
  global fndry fmly wght pxlsz ptsz slant swdth rgstry

  eval set [lowercase $w] $v
  :modify $w :value $v
}

proc click-select {} {
  global widgets wtype
  global fndrys fmlys wghts pxlszs ptszs slants swdths rgstrys
  global fndry fmly wght pxlsz ptsz slant swdth rgstry

  set widget [lindex $widgets $wtype]
  if {$wght == 2} then {set comp "b"} else {set comp "n"}

  :font local $widget $comp          \
     :fndry [lindex $fndrys $fndry]  \
     :fmly  [lindex $fmlys $fmly]    \
     :wght  [lindex $wghts $wght]    \
     :pxlsz [lindex $pxlszs $pxlsz]  \
     :ptsz  [lindex $ptszs $ptsz]    \
     :slant [lindex $slants $slant]  \
     :swdth [lindex $swdths $swdth]  \
     :rgstry [lindex $rgstrys $rgstry]

  paint-font
  paint-sample
}

proc click-fontselect {v} {
  global widgets wtype
  global wght

  set widget [lindex $widgets $wtype]
  if {$wght == 2} then {set weight "b"} else {set weight "n"}

  global _font_[set widget]_[set weight]
  set _font_[set widget]_[set weight] "$v"

  paint-font
  paint-sample
}

proc click-restore {} {
   global env fixed

   click-reset

if {[info exists env(tkfixedfont)]} then {set fixed $env(tkfixedfont)} else {set fixed "courier"}

  switch $env(os)  {
      Win {
         if {[info exists env(FONTFMLY)]}  then {set fmly $env(FONTFMLY)} else {set fmly "clean"}
         if {[info exists env(FONTSIZE)]}  then {set size $env(FONTSIZE)} else {set size "medium"}
      }
      SunOS  {
         if {[info exists env(FONTFMLY)]}  then {set fmly  $env(FONTFMLY)}   else {set fmly "times"}
         if {[info exists env(FONTSIZE)]}  then {set size  $env(FONTSIZE)}   else {set size "medium"}
      }
      Linux64  -
      Linux   {
         if {[info exists env(FONTFMLY)]}  then {set fmly  $env(FONTFMLY)}   else {set fmly "times"}
         if {[info exists env(FONTSIZE)]}  then {set size  $env(FONTSIZE)}   else {set size "medium"}
      }
   }

   font-config_8x  "$fmly" $size

   inform root exec-font-defaults
   font-setup
   paint-sample
}

proc click-reset {} {
  global env widgets

   foreach widget $widgets {
     foreach comp {n b} {
       global _font_[set widget]_[set comp]
       catch {unset _font_[set widget]_[set comp]}
     }
   }
   paint-sample
}

proc click-apply {} {
  global env widgets

   foreach widget $widgets {
     foreach comp {n b} {
       global _font_[set widget]_[set comp]
       if { [info exists _font_[set widget]_[set comp]] } then {
         set env(tkfont$widget$comp) [set _font_[set widget]_[set comp]]
       }
     }
   }
   font-setup
}

proc click-save {} {
  global env savefile widgets
  set z [catch {open $savefile w} file]
  if {$z != 0} then {set-footer $file; return}
  foreach widget $widgets {
    foreach comp {n b} {
      global _font_[set widget]_[set comp]
      if { [info exists _font_[set widget]_[set comp]] } then {
        set font [set _font_[set widget]_[set comp]]
      } else {
        if {[info exists env(tkfont$widget$comp)]} then {
          set font $env(tkfont$widget$comp)
        } else {
          set font ""
        }
      }
    if {$font != ""} then {puts $file "set env(tkfont$widget$comp) \"$font\""}
    }
  }
  close $file
}

proc click arg {
   global debug
   global fndry fmly wght pxlsz ptsz slant swdth rgstry
   global wtype

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      SELECT    {click-select}
      FVALUE    {click-fontselect $v}

      WTYPE     {set wtype $v; :modify $w :value $v}

      FNDRY     -
      FMLY      -
      WGHT      -
      PXLSZ     -
      PTSZ      -
      SLANT     -
      SWDTH     -
      RGSTRY    {click-font $w $v}

      RESTORE   {click-restore}
      SAVE      {click-save}
      RESET     {click-reset}
      APPLY     {click-apply}
      REDISPLAY {paint-sample}

      default   {}
   }
   paint-font
}


proc start args {
   global started 
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   paint-font
   paint-sample
}
