# poly.tcl---polygon array editor

# Tue Apr 19 13:54:56 BST 1994, DB  (copied version)
#   edited 16.7.97 to use :spectrum interface


set polyarray(selected) {}
set polyarray(polys)    {}
set polyarray(name)  "demo"

proc poly-xys p             {global $p; set ${p}(xys)}
proc poly-parent p          {global $p; set ${p}(parent)}
proc set-poly-number {p n}  {global $p; set ${p}(number) $n}
proc set-poly-xys {p xys}   {global $p; set ${p}(xys) $xys}
proc set-poly-parent {p pp} {global $p; set ${p}(parent) $pp}
proc poly-order p           {global $p; llength [set ${p}(xys)]}
proc poly-number p          {global $p; set ${p}(number)}
proc make-poly p            {global $p; set ${p}(xys) {}}

proc poly-status p {
   global $p
   set n [poly-order $p]
   if {$n == 0} then {
      set s "null polygon"
   } else {
      set s "[expr $n/2] vertices"
   }
   return $s
}

proc polyarray-selected pl {global $pl; set ${pl}(selected)}
proc polyarray-select {pl p} {global $pl; set ${pl}(selected) $p}
proc polyarray-deselect {pl p} {global $pl; set ${pl}(selected) {}}

proc polyarray-next {pl p} {
   global $pl
   set ps [set ${pl}(polys)]
   if {$p != ""} then {set i [lsearch -exact $ps $p]} else {set i 0}
   set n [llength $ps]
   set i [expr ($i+1)%$n]
   lindex $ps $i
}

proc draw-poly p {
   global gallery
   if {[poly-order $p] == 0} then return
   set xys [poly-xys $p]
   :spectrum $gallery gate set2d $xys
}

proc redraw-poly p {draw-poly $p}

proc get-poly-name n {return [join [front [split $n "-"]] "-"]}

proc poly-name n {
   global polyarray
   return $polyarray(name)-[last [split $n "-"]]
}

proc deselect-poly p {
    global gallery
    set pl [poly-parent $p]
    set-poly-xys $p [:spectrum $gallery gate get2d]
    polyarray-deselect $pl $p
}

proc select-poly p {
   global  gallery
   set pl [poly-parent $p]
   polyarray-select $pl $p
   :spectrum $gallery gate select [poly-number $p]
   set-poly-xys $p [:spectrum $gallery gate get2d]
   set-footer "selected polygon is [poly-name $p] ([poly-status $p])"
}

proc select-poly-by-number n {
   set p "pa-p$n"
   select-poly $p
}

proc delete-polyarray pa {
   global $pa gallery
   set ps [polyarray-selected $pa]
   if {"$ps" != ""} then {deselect-poly $ps}
   foreach p [set ${pa}(polys)] {
       set i [lsearch -exact $polyarray(polys) "$p"]
       set polyarray(polys) [lreplace $polyarray(polys)[set polyarray(polys) {}] $i $i]
       set n [poly-number $p]
       :spectrum $gallery gate set2d {} $n
   }
}

proc adjacent? {p is} {
   set n [poly-order $p]
   set d [expr [second $is]-[first $is]]
   if {$d < 0} then {incr d $n}
   expr {$d == 1 || $d+1 == $n}
}

proc make-vertices p {
    global gallery
    set vs {}
    :spectrum $gallery gate select [poly-number $p]
    set-poly-xys $p [:spectrum $gallery gate get2d]
    set n [poly-order $p]
    set xys [poly-xys $p]
    for {set i 0} {$i<$n} {incr i 2} {
	lappend vs [list [lindex $xys $i] [lindex $xys [expr $i + 1]]]
    }
    return $vs
}

#    "add vertex" menu item
proc click-add-vertex {} {
    global  gallery
    :spectrum $gallery gate vertex add
}

#    "delete vertex" menu item
proc click-delete-vertex {} {
    global  gallery 
    :spectrum $gallery gate vertex remove
}


#    "select next polygon" menu item
proc click-next-polygon {} {
   global polyarray
   if {[llength $polyarray(polys)] == 0} then {return}
   set p [polyarray-selected polyarray]
   if {"$p" != ""} then {deselect-poly $p}
   set p [polyarray-next polyarray $p]
   select-poly $p
}

#    "add polygon from tags" menu item
proc click-add-polygon {} {
    global gallery polyarray
    set xys [:spectrum $gallery tag get2d -1]
    if {[llength $xys] < 3} then {
	set-footer "Too few tags defined ... need at least 3! ..."
	return
    }
    :spectrum $gallery tag pop -1
    set i [llength $polyarray(polys)]
    :spectrum $gallery gate set2d $xys $i
    set p "pa-p$i"
    make-poly $p
    set-poly-xys $p $xys
    set-poly-number $p $i
    set-poly-parent $p polyarray
    lappend polyarray(polys) $p
# now update selected polygon ...
    set pold [polyarray-selected polyarray]
    if {"$pold" != ""} then {deselect-poly $pold}
    select-poly $p
}

#    "delete polygon" menu item
proc click-delete-polygon {w} {
    global  gallery
    global polyarray
    if {$w == "selected"} then {
       set p [polyarray-selected polyarray]
       if {"$p" == ""} then {
          set-footer "You haven't selected a polygon yet!!!"
          return
       }
       set i [lsearch -exact $polyarray(polys) "$p"]
       set polyarray(polys) [lreplace $polyarray(polys)[set polyarray(polys) {}] $i $i]
       set n [poly-number $p]
    } else {
       if {$w < 0} then {
#    delete all
          set polyarray(polys) {}
       } else {
#    delete specific
          set i [lsearch -exact $polyarray(polys) "pa-p$w"]
          set polyarray(polys) [lreplace $polyarray(polys)[set polyarray(polys) {}] $i $i]
       }
       set n $w
    }
    :spectrum $gallery gate set2d {} $n
    if {[llength $polyarray(polys)] > 0} then {
       select-poly [first $polyarray(polys)]
    }
}

#   "project spectrum" menu item
proc click-project-spectrum {} {
   global polyarray
   set p [polyarray-selected polyarray]
   if {"$p" == ""} then {
      set-footer "You haven't selected a polygon yet!!!"
      return
   }
   poly-project-current [make-vertices $p]   
}


#    "directory browser" menu item
proc click-directory-browser {} {inform pdb start}

#    procedures called from pdb

proc load-polyarray-file {dir file} {
   global frame  polyarray 
#   delete-polyarray polyarray
   click-delete-polygon -1
   set polyarray(frame) $frame
   set polyarray(selected) {}
   set i 0; set ps {}
   set z [catch {set polys [read-polyarray-file [file join $dir $file]]} m]
   if $z!=0 then {set-footer $m; return}
   foreach xys $polys {
      set p "pa-p$i"
      make-poly $p
      set-poly-xys $p $xys
      set-poly-number $p $i
      set-poly-parent $p polyarray
      draw-poly $p
      lappend ps $p
      incr i
   }
   set polyarray(polys) $ps
   set polyarray(name) $file
   select-poly [first $ps]
}

proc save-polyarray-file {dir file interps} {
   global polyarray
   set polys {}
   foreach p $polyarray(polys) {lappend polys [make-vertices $p]}
   write-polyarray-file [file join $dir $file] $polys
   send $interps click-redisplay
}

proc read-polyarray-file fn {
   set z [catch {open $fn r} f]
   if {$z != 0} then {
       midas-warning "Error reading polyarray file\n$f"
       error $f
   }
   set data [read $f]
   close $f

   if {[llength [first [first $data]]] == 2} then {
      set polys $data
   } else {
      set len [expr ([llength $data] / 2) *2]
      if {$len != [llength $data]} then {
         midas-warning "Warning - there more x's than y's"
      }
      set polys {}
      set xys {}
      for {set i 0} {$i < $len} {incr i 2} {
         set item [lindex $data $i]
         lappend item [lindex $data [expr $i +1]]
         lappend xys $item
      }
      lappend polys $xys
   }

   return $polys
}

proc write-polyarray-file {fn polys} {
   set z [catch {open $fn w} f]
   if {$z != 0} then {
       midas-warning "Error saving polyarray file\n$f"
       error $f
   }
   puts $f $polys
   close $f
}

